@@ -5,12 +5,18 @@ import nipype.pipeline.engine as pe
 import nipype.interfaces.fsl as fsl
 import nipype.interfaces.ants as ants
 
+# TODO
+# from nipype.interfaces.niftyreg import reg
+
+# TODO
+# from nipype.interfaces.niftyreg import reg
+
 from nipype.interfaces.ants.segmentation import DenoiseImage
 
 from ..utils.utils_nodes import NodeParams, MapNodeParams
 from ..utils.misc import parse_key
 
-from ..nodes.prepare import average_align, FslOrient
+from ..nodes.prepare import average_align, FslOrient, reg_aladin_dirty
 from ..nodes.extract_brain import T1xT2BET
 
 
@@ -85,17 +91,9 @@ def _create_prep_pipeline(params, name="prep_pipeline", node_suffix=""):
     Params:
 
     - reorient (new dims as a string (e.g. "x z -y"))
-    - denoise_first (see `DenoiseImage <https://nipype.readthedocs.io/en/0.12.\
-    1/interfaces/generated/nipype.interfaces.ants.segmentation.html\
-    #denoiseimage>`_ for arguments)) - also available \
-    as :ref:`indiv_params <indiv_params>`
     - crop (see `ExtractROI <https://nipype.readthedocs.io/en/0.12.1/\
     interfaces/generated/nipype.interfaces.fsl.utils.html#extractroi>`_ \
     for arguments) - also available as :ref:`indiv_params <indiv_params>`
-    - norm_intensity (see `N4BiasFieldCorrection <https://nipype.readthedocs.\
-    io/en/0.12.1/interfaces/generated/nipype.interfaces.ants.segmentation.\
-    html#n4biasfieldcorrection>`_ for arguments) - also available as \
-    :ref:`indiv_params <indiv_params>`
     - denoise (see `DenoiseImage <https://nipype.readthedocs.io/en/0.12.\
     1/interfaces/generated/nipype.interfaces.ants.segmentation.html\
     #denoiseimage>`_ for arguments)) - also available \
@@ -125,32 +123,18 @@ def _create_prep_pipeline(params, name="prep_pipeline", node_suffix=""):
         prep_pipeline.connect(inputnode, 'img',
                               reorient_pipe, 'inputnode.image')
 
-    if "denoise_first" in params.keys():
-        # denoise with Ants package
-        # (taking into account whether reorient has been performed or not)
-        denoise_first = NodeParams(interface=DenoiseImage(),
-                                   params=parse_key(params, "denoise_first"),
-                                   name="denoise")
-
-        if "reorient" in params.keys():
-            prep_pipeline.connect(reorient_pipe, 'reorient.out_file',
-                                  denoise_first, 'input_image')
-        else:
-            prep_pipeline.connect(inputnode, 'img',
-                                  denoise_first, 'input_image')
-        prep_pipeline.connect(
-            inputnode, ('indiv_params', parse_key, "denoise_first"),
-            denoise_first, 'indiv_params')
-
     # cropping
+    if 'crop'+node_suffix not in params.keys():
+        print("Error, crop"+node_suffix+" could be found in params")
+        print("Breaking")
+
+        exit(0)
+
     crop = NodeParams(fsl.ExtractROI(),
                       params=parse_key(params, "crop"+node_suffix),
                       name='crop'+node_suffix)
 
-    if "denoise_first" in params.keys():
-        prep_pipeline.connect(denoise_first, 'output_image',
-                              crop, 'in_file')
-    elif "reorient" in params.keys():
+    if "reorient" in params.keys():
         prep_pipeline.connect(reorient_pipe, 'reorient.out_file',
                               crop, 'in_file')
     else:
@@ -211,17 +195,9 @@ def _create_mapnode_prep_pipeline(params, name="mapnode_prep_pipeline",
     Params: (Warning: **all arguments are in lists**)
 
     - reorient (new dims as a string (e.g. "x z -y"))
-    - denoise_first (see `DenoiseImage <https://nipype.readthedocs.io/en/0.12.\
-    1/interfaces/generated/nipype.interfaces.ants.segmentation.html\
-    #denoiseimage>`_ for arguments)) - also available \
-    as :ref:`indiv_params <indiv_params>`
     - crop (see `ExtractROI <https://nipype.readthedocs.io/en/0.12.1/\
     interfaces/generated/nipype.interfaces.fsl.utils.html#extractroi>`_ \
     for arguments) - also available as :ref:`indiv_params <indiv_params>`
-    - norm_intensity (see `N4BiasFieldCorrection <https://nipype.readthedocs.\
-    io/en/0.12.1/interfaces/generated/nipype.interfaces.ants.segmentation.\
-    html#n4biasfieldcorrection>`_ for arguments) - also available as \
-    :ref:`indiv_params <indiv_params>`
     - denoise (see `DenoiseImage <https://nipype.readthedocs.io/en/0.12.\
     1/interfaces/generated/nipype.interfaces.ants.segmentation.html\
     #denoiseimage>`_ for arguments)) - also available \
@@ -251,34 +227,18 @@ def _create_mapnode_prep_pipeline(params, name="mapnode_prep_pipeline",
             mapnode_prep_pipeline.connect(inputnode, 'list_img',
                                           reorient_pipe, 'inputnode.list_img')
 
-    if "denoise_first" in params.keys():
+    if 'crop'+node_suffix not in params.keys():
+        print("Error, crop"+node_suffix+" could be found in params")
+        print("Breaking")
 
-        # denoise_first with Ants package
-        # (taking into account whether reorient has been performed or not)
-        denoise_first = MapNodeParams(
-            interface=DenoiseImage(),
-            params=parse_key(params, "denoise_first"), name="denoise_first",
-            iterfield=["input_image"])
-
-        if "reorient" in params.keys():
-            mapnode_prep_pipeline.connect(reorient_pipe, 'reorient.out_file',
-                                          denoise_first, 'input_image')
-        else:
-            mapnode_prep_pipeline.connect(inputnode, 'list_img',
-                                          denoise_first, 'input_image')
-        mapnode_prep_pipeline.connect(
-            inputnode, ("indiv_params", parse_key, "denoise_first"),
-            denoise_first, 'indiv_params')
+        exit(0)
 
     # cropping
     crop = MapNodeParams(fsl.ExtractROI(),
                          params=parse_key(params, "crop"+node_suffix),
                          name='crop', iterfield=["in_file", "args"])
 
-    if "denoise_first" in params.keys():
-        mapnode_prep_pipeline.connect(denoise_first, 'output_image',
-                                      crop, 'in_file')
-    elif "reorient" in params.keys():
+    if "reorient" in params.keys():
         mapnode_prep_pipeline.connect(reorient_pipe, 'reorient.out_file',
                                       crop, 'in_file')
     else:
@@ -399,6 +359,11 @@ def create_short_preparation_pipe(params, name="short_preparation_pipe"):
         name='inputnode'
     )
 
+    # Creating output node
+    outputnode = pe.Node(
+        niu.IdentityInterface(fields=['preproc_T1', 'preproc_T2']),
+        name='outputnode')
+
     # avererge if multiple T2
     av_T2 = pe.Node(niu.Function(
         input_names=['list_img'],
@@ -509,35 +474,47 @@ def create_short_preparation_pipe(params, name="short_preparation_pipe"):
                                           bet_crop, 't2_file')
 
     # denoise with Ants package
-    denoise_T1 = NodeParams(interface=DenoiseImage(),
-                            params=parse_key(params, "denoise"),
-                            name="denoise_T1")
-    denoise_T2 = NodeParams(interface=DenoiseImage(),
-                            params=parse_key(params, "denoise"),
-                            name="denoise_T2")
+    if "denoise" in params.keys():
 
-    if "crop_T1" in params.keys():
-        data_preparation_pipe.connect(crop_T1, "roi_file",
-                                      denoise_T1, 'input_image')
-        data_preparation_pipe.connect(crop_T2, "roi_file",
-                                      denoise_T2, 'input_image')
+        denoise_T1 = NodeParams(interface=DenoiseImage(),
+                                params=parse_key(params, "denoise"),
+                                name="denoise_T1")
 
+        denoise_T2 = NodeParams(interface=DenoiseImage(),
+                                params=parse_key(params, "denoise"),
+                                name="denoise_T2")
+        # inputs
+        if "crop_T1" in params.keys():
+            data_preparation_pipe.connect(crop_T1, "roi_file",
+                                          denoise_T1, 'input_image')
+            data_preparation_pipe.connect(crop_T2, "roi_file",
+                                          denoise_T2, 'input_image')
+
+        else:
+            data_preparation_pipe.connect(bet_crop, "t1_cropped_file",
+                                          denoise_T1, 'input_image')
+            data_preparation_pipe.connect(bet_crop, "t2_cropped_file",
+                                          denoise_T2, 'input_image')
+
+        # outputs
+        data_preparation_pipe.connect(denoise_T1, 'output_image',
+                                      outputnode, 'preproc_T1')
+
+        data_preparation_pipe.connect(denoise_T2, 'output_image',
+                                      outputnode, 'preproc_T2')
     else:
-        data_preparation_pipe.connect(bet_crop, "t1_cropped_file",
-                                      denoise_T1, 'input_image')
-        data_preparation_pipe.connect(bet_crop, "t2_cropped_file",
-                                      denoise_T2, 'input_image')
 
-    # Creating output node
-    outputnode = pe.Node(
-        niu.IdentityInterface(fields=['preproc_T1', 'preproc_T2']),
-        name='outputnode')
+        if "crop_T1" in params.keys():
+            data_preparation_pipe.connect(crop_T1, "roi_file",
+                                          outputnode, 'preproc_T1')
+            data_preparation_pipe.connect(crop_T2, "roi_file",
+                                          outputnode, 'preproc_T2')
 
-    data_preparation_pipe.connect(denoise_T1, 'output_image',
-                                  outputnode, 'preproc_T1')
-
-    data_preparation_pipe.connect(denoise_T2, 'output_image',
-                                  outputnode, 'preproc_T2')
+        else:
+            data_preparation_pipe.connect(bet_crop, "t1_cropped_file",
+                                          outputnode, 'preproc_T1')
+            data_preparation_pipe.connect(bet_crop, "t2_cropped_file",
+                                          outputnode, 'preproc_T2')
 
     return data_preparation_pipe
 
@@ -736,8 +713,8 @@ def create_long_multi_preparation_pipe(params,
 
     # mapnode_prep_pipeline for T1 list
     mapnode_prep_T1_pipe = _create_mapnode_prep_pipeline(
-        params=parse_key(params, "mapnode_prep_T1"),
-        name="mapnode_prep_T1", node_suffix="_T1")
+        params=parse_key(params, "prep_T1"),
+        name="prep_T1", node_suffix="_T1")
 
     long_multi_preparation_pipe.connect(
         inputnode, 'list_T1', mapnode_prep_T1_pipe, "inputnode.list_img")
@@ -758,8 +735,8 @@ def create_long_multi_preparation_pipe(params,
 
     # mapnode_prep_pipeline for T2 list
     mapnode_prep_T2_pipe = _create_mapnode_prep_pipeline(
-        params=parse_key(params, "mapnode_prep_T2"),
-        name="mapnode_prep_T2", node_suffix="_T2")
+        params=parse_key(params, "prep_T2"),
+        name="prep_T2", node_suffix="_T2")
 
     long_multi_preparation_pipe.connect(
         inputnode, 'list_T2', mapnode_prep_T2_pipe, "inputnode.list_img")
@@ -1000,17 +977,42 @@ def create_short_preparation_FLAIR_pipe(params,
         name='inputnode'
     )
 
-    # align FLAIR on avg T1
-    align_FLAIR_on_T1 = pe.Node(fsl.FLIRT(), name="align_FLAIR_on_T1")
-    align_FLAIR_on_T1.inputs.dof = 6
-    align_FLAIR_on_T1.inputs.cost = "mutualinfo"
-    align_FLAIR_on_T1.inputs.cost_func = "mutualinfo"
+    if "align_FLAIR_on_T1" in params.keys():
+        print("in align_FLAIR_on_T1")
 
-    data_preparation_pipe.connect(inputnode, 'orig_T1',
-                                  align_FLAIR_on_T1, 'reference')
+        # align FLAIR on avg T1
+        align_FLAIR_on_T1 = NodeParams(
+            fsl.FLIRT(),
+            params=parse_key(params, "align_FLAIR_on_T1"),
+            name="align_FLAIR_on_T1")
 
-    data_preparation_pipe.connect(inputnode, 'FLAIR',
-                                  align_FLAIR_on_T1, 'in_file')
+        data_preparation_pipe.connect(inputnode, 'orig_T1',
+                                      align_FLAIR_on_T1, 'reference')
+
+        data_preparation_pipe.connect(inputnode, 'FLAIR',
+                                      align_FLAIR_on_T1, 'in_file')
+
+    elif "reg_aladin_FLAIR_on_T1" in params.keys():
+        print("in reg_aladin_FLAIR_on_T1")
+        align_FLAIR_on_T1 = pe.Node(
+            niu.Function(
+                input_names=["reference", "in_file"],
+                output_names=["out_file"],
+                function=reg_aladin_dirty),
+            name="reg_aladin_FLAIR_on_T1")
+
+        # TODO
+        # align_FLAIR_on_T1 = pe.Node(reg.RegAladin(),
+        # name="reg_aladin_FLAIR_on_T1")
+
+        data_preparation_pipe.connect(inputnode, 'orig_T1',
+                                      align_FLAIR_on_T1, 'reference')
+
+        data_preparation_pipe.connect(inputnode, 'FLAIR',
+                                      align_FLAIR_on_T1, 'in_file')
+    else:
+        print("no align_FLAIR_on_T1 or reg_aladin_FLAIR_on_T1, breaking")
+        exit(0)
 
     # Creating output node
     outputnode = pe.Node(
@@ -1080,25 +1082,25 @@ def create_short_preparation_MD_pipe(params,
 
     # Creating input node
     inputnode = pe.Node(
-        niu.IdentityInterface(fields=['orig_T2', 'SS_T2', 'MD',
+        niu.IdentityInterface(fields=['SS_T2', 'MD',
                                       'b0mean', 'native_wm_mask']),
         name='inputnode'
     )
 
     # init_align_b0mean_on_T2
-    init_align_b0mean_on_T2 = pe.Node(fsl.FLIRT(),
-                                      name="init_align_b0mean_on_T2")
-    init_align_b0mean_on_T2.inputs.dof = 6
+    init_align_b0mean_on_T2 = NodeParams(
+        fsl.FLIRT(), params=parse_key(params, "init_align_b0mean_on_T2"),
+        name="init_align_b0mean_on_T2")
 
-    data_preparation_pipe.connect(inputnode, 'orig_T2',
+    data_preparation_pipe.connect(inputnode, 'SS_T2',
                                   init_align_b0mean_on_T2, 'reference')
     data_preparation_pipe.connect(inputnode, 'b0mean',
                                   init_align_b0mean_on_T2, 'in_file')
 
     # align_b0mean_on_T2
-    align_b0mean_on_T2 = pe.Node(fsl.FLIRT(), name="align_b0mean_on_T2")
-    align_b0mean_on_T2.inputs.dof = 6
-    align_b0mean_on_T2.inputs.cost = "bbr"
+    align_b0mean_on_T2 = NodeParams(
+        fsl.FLIRT(), params=parse_key(params, "align_b0mean_on_T2"),
+        name="align_b0mean_on_T2")
 
     data_preparation_pipe.connect(inputnode, 'SS_T2',
                                   align_b0mean_on_T2, 'reference')
