@@ -1,3 +1,5 @@
+import shutil
+
 import nipype.interfaces.utility as niu
 import nipype.pipeline.engine as pe
 
@@ -6,11 +8,11 @@ import nipype.interfaces.afni as afni
 import nipype.interfaces.ants as ants
 
 from ..utils.misc import get_elem
-from ..nodes.segment import split_indexed_mask
 from ..utils.utils_nodes import NodeParams, parse_key
 
 from ..nodes.register import (interative_flirt, NMTSubjectAlign,
-                              NMTSubjectAlign2, NwarpApplyPriors)
+                              NMTSubjectAlign2, NwarpApplyPriors,
+                              animal_warper)
 
 
 def create_iterative_register_pipe(
@@ -201,41 +203,79 @@ def create_register_NMT_pipe(params_template, params={},
     if "NMT_version" in params.keys():
         NMT_version = params['NMT_version']
 
-        print("*** Overriding NMT_version with parmas {}".format(
+        print("*** Overriding NMT_version with params NMT_version: {}".format(
             params['NMT_version']))
 
-    # N4 intensity normalization over brain
-    norm_intensity = NodeParams(ants.N4BiasFieldCorrection(),
-                                params=parse_key(params, "norm_intensity"),
-                                name='norm_intensity')
+    if "norm_intensity" in params.keys():
 
-    register_NMT_pipe.connect(inputnode, 'T1',
-                              norm_intensity, "input_image")
+        # N4 intensity normalization over brain
+        norm_intensity = NodeParams(ants.N4BiasFieldCorrection(),
+                                    params=parse_key(params, "norm_intensity"),
+                                    name='norm_intensity')
 
-    register_NMT_pipe.connect(
-        inputnode, ('indiv_params', parse_key, "norm_intensity"),
-        norm_intensity, "indiv_params")
+        register_NMT_pipe.connect(inputnode, 'T1',
+                                  norm_intensity, "input_image")
 
+        register_NMT_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "norm_intensity"),
+            norm_intensity, "indiv_params")
+
+    # deoblique (seems mandatory from NMTSubjectAlign)
     deoblique = pe.Node(afni.Refit(deoblique=True), name="deoblique")
-    register_NMT_pipe.connect(norm_intensity, 'output_image',
-                              deoblique, "in_file")
 
-    print("*** Found NMT_version {}".format(NMT_version))
+    if "norm_intensity" in params.keys():
 
-    if NMT_version == "v1.2":
-        # align subj to nmt
-        NMT_subject_align = NodeParams(
-            NMTSubjectAlign(), params=parse_key(params, "NMT_subject_align"),
-            name='NMT_subject_align')
-
-    elif NMT_version == "v1.3" or NMT_version == "v2.0":
-        # align subj to nmt
-        NMT_subject_align = NodeParams(
-            NMTSubjectAlign2(), params=parse_key(params, "NMT_subject_align"),
-            name='NMT_subject_align')
+        register_NMT_pipe.connect(norm_intensity, 'output_image',
+                                  deoblique, "in_file")
 
     else:
-        print("NMT_version {} is not implemented".format(NMT_version))
+
+        register_NMT_pipe.connect(inputnode, 'T1',
+                                  deoblique, "in_file")
+
+    print("which @animal_warper: ", shutil.which("@animal_warper"))
+
+    if "NMT_subject_align" in params.keys() \
+            or shutil.which("@animal_warper") is None:
+
+        print("running NMT_subject_align with version {}".format(NMT_version))
+
+        if NMT_version == "v1.2":
+            # align subj to nmt
+            NMT_subject_align = NodeParams(
+                NMTSubjectAlign(),
+                params=parse_key(params, "NMT_subject_align"),
+                name='NMT_subject_align')
+
+        elif NMT_version == "v1.3" or NMT_version == "v2.0":
+            # align subj to nmt
+            NMT_subject_align = NodeParams(
+                NMTSubjectAlign2(),
+                params=parse_key(params, "NMT_subject_align"),
+                name='NMT_subject_align')
+
+        else:
+            print("NMT_version {} is not implemented, breaking".format(
+                NMT_version))
+
+            exit()
+
+    elif shutil.which("@animal_warper") is not None:
+        # TODO AnimalWarper()
+
+        print("running @animal_warper with version {}".format(
+            shutil.which("@animal_warper")))
+
+        NMT_subject_align = pe.Node(
+            niu.Function(input_names=["T1_file", "NMT_SS_file"],
+                         output_names=["aff_file", "warp_file",
+                                       "warpinv_file", "transfo_file",
+                                       "inv_transfo_file"],
+                         function=animal_warper),
+            name='NMT_subject_align')
+    else:
+        print("could not find normalisation procedure,\
+              either with NMT_subject_align, or @animal_warper, breaking")
         exit()
 
     NMT_subject_align.inputs.NMT_SS_file = params_template["template_brain"]
@@ -279,8 +319,10 @@ def create_register_NMT_pipe(params_template, params={},
 
     register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 0),
                               align_NMT, "in_file")  # -source
-    register_NMT_pipe.connect(norm_intensity, 'output_image',
+
+    register_NMT_pipe.connect(deoblique, 'out_file',
                               align_NMT, "reference")  # -base
+
     register_NMT_pipe.connect(NMT_subject_align, 'inv_transfo_file',
                               align_NMT, "in_matrix")  # -1Dmatrix_apply
 
@@ -295,8 +337,10 @@ def create_register_NMT_pipe(params_template, params={},
 
         register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 1),
                                   align_seg_csf, "in_file")  # -source
-        register_NMT_pipe.connect(norm_intensity, 'output_image',
+
+        register_NMT_pipe.connect(deoblique, 'out_file',
                                   align_seg_csf, "reference")  # -base
+
         register_NMT_pipe.connect(
             NMT_subject_align, 'inv_transfo_file', align_seg_csf,
             "in_matrix")  # -1Dmatrix_apply
@@ -310,7 +354,7 @@ def create_register_NMT_pipe(params_template, params={},
 
         register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 2),
                                   align_seg_gm, "in_file")  # -source
-        register_NMT_pipe.connect(norm_intensity, 'output_image',
+        register_NMT_pipe.connect(deoblique, 'out_file',
                                   align_seg_gm, "reference")  # -base
         register_NMT_pipe.connect(NMT_subject_align, 'inv_transfo_file',
                                   align_seg_gm, "in_matrix")  # -1Dmatrix_apply
@@ -324,7 +368,7 @@ def create_register_NMT_pipe(params_template, params={},
 
         register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 3),
                                   align_seg_wm, "in_file")  # -source
-        register_NMT_pipe.connect(norm_intensity, 'output_image',
+        register_NMT_pipe.connect(deoblique, 'out_file',
                                   align_seg_wm, "reference")  # -base
         register_NMT_pipe.connect(NMT_subject_align, 'inv_transfo_file',
                                   align_seg_wm, "in_matrix")  # -1Dmatrix_apply
@@ -340,7 +384,7 @@ def create_register_NMT_pipe(params_template, params={},
 
         register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 1),
                                   align_seg, "in_file")  # -source
-        register_NMT_pipe.connect(norm_intensity, 'output_image',
+        register_NMT_pipe.connect(deoblique, 'out_file',
                                   align_seg, "reference")  # -base
         register_NMT_pipe.connect(NMT_subject_align, 'inv_transfo_file',
                                   align_seg, "in_matrix")  # -1Dmatrix_apply
@@ -350,12 +394,19 @@ def create_register_NMT_pipe(params_template, params={},
 
 def create_reg_seg_pipe(name="reg_seg_pipe"):
 
-    reg_seg_pipe = pe.Workflow(name=name)
+    reg_pipe = pe.Workflow(name=name)
 
     # creating inputnode
     inputnode = pe.Node(
-        niu.IdentityInterface(fields=['native_segmented_file',
-                                      'transfo_file', 'ref_image']),
+        niu.IdentityInterface(fields=['native_seg',
+                                      'native_threshold_gm',
+                                      'native_threshold_wm',
+                                      'native_threshold_csf',
+                                      'native_prob_gm',
+                                      'native_prob_wm',
+                                      'native_prob_csf',
+                                      'transfo_file',
+                                      'ref_image']),
         name='inputnode')
 
     # align_seg
@@ -365,32 +416,105 @@ def create_reg_seg_pipe(name="reg_seg_pipe"):
     align_seg.inputs.overwrite = True
     align_seg.inputs.outputtype = "NIFTI_GZ"
 
-    reg_seg_pipe.connect(inputnode, 'native_segmented_file',
-                         align_seg, "in_file")
-    reg_seg_pipe.connect(inputnode, 'ref_image', align_seg, "reference")
-    reg_seg_pipe.connect(inputnode, 'transfo_file', align_seg, "in_matrix")
+    reg_pipe.connect(inputnode, 'native_seg',
+                     align_seg, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_seg, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file', align_seg, "in_matrix")
 
-    # align_gm
-    split_seg_mask = pe.Node(
-        niu.Function(input_names=['nii_file'],
-                     output_names=['list_binary_masks'],
-                     function=split_indexed_mask),
-        name="split_seg_mask")
+    # align_threshold_gm
+    align_threshold_gm = pe.Node(
+        afni.Allineate(), name="align_threshold_gm", iterfield=['in_file'])
+    align_threshold_gm.inputs.final_interpolation = "nearestneighbour"
+    align_threshold_gm.inputs.overwrite = True
+    align_threshold_gm.inputs.outputtype = "NIFTI_GZ"
 
-    reg_seg_pipe.connect(align_seg, 'out_file',
-                         split_seg_mask, "nii_file")
+    reg_pipe.connect(inputnode, 'native_threshold_gm',
+                     align_threshold_gm, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_threshold_gm, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file',
+                     align_threshold_gm, "in_matrix")
+
+    # align_threshold_wm
+    align_threshold_wm = pe.Node(
+        afni.Allineate(), name="align_threshold_wm", iterfield=['in_file'])
+    align_threshold_wm.inputs.final_interpolation = "nearestneighbour"
+    align_threshold_wm.inputs.overwrite = True
+    align_threshold_wm.inputs.outputtype = "NIFTI_GZ"
+
+    reg_pipe.connect(inputnode, 'native_threshold_wm',
+                     align_threshold_wm, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_threshold_wm, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file',
+                     align_threshold_wm, "in_matrix")
+
+    # align_threshold_csf
+    align_threshold_csf = pe.Node(
+        afni.Allineate(), name="align_threshold_csf", iterfield=['in_file'])
+    align_threshold_csf.inputs.final_interpolation = "nearestneighbour"
+    align_threshold_csf.inputs.overwrite = True
+    align_threshold_csf.inputs.outputtype = "NIFTI_GZ"
+
+    reg_pipe.connect(inputnode, 'native_threshold_csf',
+                     align_threshold_csf, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_threshold_csf, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file',
+                     align_threshold_csf, "in_matrix")
+
+    # align_prob_gm
+    align_prob_gm = pe.Node(
+        afni.Allineate(), name="align_prob_gm", iterfield=['in_file'])
+    align_prob_gm.inputs.final_interpolation = "nearestneighbour"
+    align_prob_gm.inputs.overwrite = True
+    align_prob_gm.inputs.outputtype = "NIFTI_GZ"
+
+    reg_pipe.connect(inputnode, 'native_prob_gm',
+                     align_prob_gm, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_prob_gm, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file', align_prob_gm, "in_matrix")
+
+    # align_prob_wm
+    align_prob_wm = pe.Node(
+        afni.Allineate(), name="align_prob_wm", iterfield=['in_file'])
+    align_prob_wm.inputs.final_interpolation = "nearestneighbour"
+    align_prob_wm.inputs.overwrite = True
+    align_prob_wm.inputs.outputtype = "NIFTI_GZ"
+
+    reg_pipe.connect(inputnode, 'native_prob_wm',
+                     align_prob_wm, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_prob_wm, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file', align_prob_wm, "in_matrix")
+
+    # align_prob_csf
+    align_prob_csf = pe.Node(
+        afni.Allineate(), name="align_prob_csf", iterfield=['in_file'])
+    align_prob_csf.inputs.final_interpolation = "nearestneighbour"
+    align_prob_csf.inputs.overwrite = True
+    align_prob_csf.inputs.outputtype = "NIFTI_GZ"
+
+    reg_pipe.connect(inputnode, 'native_prob_csf',
+                     align_prob_csf, "in_file")
+    reg_pipe.connect(inputnode, 'ref_image', align_prob_csf, "reference")
+    reg_pipe.connect(inputnode, 'transfo_file', align_prob_csf, "in_matrix")
 
     outputnode = pe.Node(
-        niu.IdentityInterface(fields=['norm_seg', 'norm_gm',
-                                      'norm_wm', 'norm_csf']),
+        niu.IdentityInterface(fields=['norm_seg', 'norm_threshold_gm',
+                                      'norm_threshold_wm',
+                                      'norm_threshold_csf',
+                                      'norm_prob_gm', 'norm_prob_wm',
+                                      'norm_prob_csf']),
         name='outputnode')
 
-    reg_seg_pipe.connect(align_seg, 'out_file', outputnode, "norm_seg")
-    reg_seg_pipe.connect(split_seg_mask, ('out_file', get_elem, 0),
-                         outputnode, "norm_gm")
-    reg_seg_pipe.connect(split_seg_mask, ('out_file', get_elem, 1),
-                         outputnode, "norm_wm")
-    reg_seg_pipe.connect(split_seg_mask, ('out_file', get_elem, 2),
-                         outputnode, "norm_csf")
+    reg_pipe.connect(align_seg, 'out_file', outputnode, "norm_seg")
 
-    return reg_seg_pipe
+    reg_pipe.connect(align_threshold_gm, 'out_file',
+                     outputnode, "norm_threshold_gm")
+    reg_pipe.connect(align_threshold_wm, 'out_file',
+                     outputnode, "norm_threshold_wm")
+    reg_pipe.connect(align_threshold_csf, 'out_file',
+                     outputnode, "norm_threshold_csf")
+
+    reg_pipe.connect(align_prob_gm, 'out_file', outputnode, "norm_prob_gm")
+    reg_pipe.connect(align_prob_wm, 'out_file', outputnode, "norm_prob_wm")
+    reg_pipe.connect(align_prob_csf, 'out_file', outputnode, "norm_prob_csf")
+
+    return reg_pipe
